"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Server_1 = require("../Server");
const services = require("../services");
const chai_1 = require("chai");
const del = require("del");
const tmp = require("tmp");
const path = require("path");
describe('Default Admin Created', function () {
    let tmpDir;
    let server;
    const PORT = 6789;
    let usernamePasswordAuthService;
    before(() => __awaiter(this, void 0, void 0, function* () {
        yield del(["./data"]);
        tmpDir = tmp.dirSync();
        Realm.clearTestState();
        server = new Server_1.Server({
            port: PORT,
            logLevel: 'off',
            dataPath: tmpDir.name
        });
        usernamePasswordAuthService = new services.UsernamePasswordAuthService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key"),
        });
        server.addService(usernamePasswordAuthService);
        server.addService(new services.RealmManagementService({
            privateKeyPath: path.join(server.config.dataPath, "keys/auth.key"),
        }));
        server.addService(new services.SyncProxyService());
        server.addService(new services.SyncService({
            dataPath: path.join(server.config.dataPath, "sync"),
            publicKeyPath: path.join(server.config.dataPath, "keys/auth.pub"),
        }));
        yield server.start();
    }));
    after(() => __awaiter(this, void 0, void 0, function* () {
        if (server) {
            yield server.shutdown();
        }
    }));
    it('should create realm-admin upon autokeygen', function () {
        return __awaiter(this, void 0, void 0, function* () {
            const realmAdminUser = usernamePasswordAuthService.getUser('realm-admin');
            chai_1.expect(realmAdminUser).to.not.be.undefined;
            chai_1.expect(realmAdminUser.userId).to.eq('realm-admin');
            chai_1.expect(realmAdminUser.isAdmin).to.be.true;
            const loggedInUser = yield usernamePasswordAuthService.attemptToLogin('realm-admin', '');
            chai_1.expect(loggedInUser.userId).to.eq('realm-admin');
            chai_1.expect(loggedInUser.isAdmin).to.true;
        });
    });
});
//# sourceMappingURL=default-admin-integration-tests.spec.js.map