"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Realm = require("realm");
const fs = require("fs-extra");
const moment = require("moment");
const superagent = require("superagent");
const URI = require("urijs");
const shared_1 = require("../shared");
const Discovery_1 = require("../discovery/Discovery");
const decorators_1 = require("../decorators");
const shared_2 = require("../shared");
let userSchema = {
    name: 'User',
    primaryKey: 'userId',
    properties: {
        userId: { type: 'string', optional: false },
        isAdmin: { type: 'bool', optional: false },
        metadata: { type: 'list', objectType: 'UserMetadataRow', default: [], optional: false }
    }
};
let metadataSchema = {
    name: 'UserMetadataRow',
    properties: {
        user: { type: 'linkingObjects', objectType: 'User', property: 'metadata' },
        key: { type: 'string', optional: false },
        value: { type: 'string', optional: false }
    }
};
let AuthService = class AuthService {
    constructor(config = {}) {
        this.config = config;
    }
    authenticateOrCreateUser(body) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('You must implement the authenticate method in your subclass');
        });
    }
    discoveryReady(discovery) {
        return __awaiter(this, void 0, void 0, function* () {
            this.discovery = discovery;
            this.privateKey = yield fs.readFile(this.config.privateKeyPath, 'utf-8');
            const syncServiceHandle = yield this.discovery.find('sync');
            if (!syncServiceHandle.address || !syncServiceHandle.port)
                throw new Error('Unable to contact the SyncService named sync');
            const realmUrl = new URI(`realm://${syncServiceHandle.address}:${syncServiceHandle.port}`)
                .segment('/__admin')
                .toString();
            const adminTokenUser = Realm.Sync.User.adminUser(shared_2.generateAdminToken(this.privateKey), realmUrl);
            const realmConfig = {
                sync: {
                    url: realmUrl,
                    user: Realm.Sync.User.adminUser(shared_2.generateAdminToken(this.privateKey), realmUrl)
                },
                schema: [userSchema, metadataSchema]
            };
            this.realm = new Realm(realmConfig);
        });
    }
    authenticate(request) {
        return __awaiter(this, void 0, void 0, function* () {
            if (request.body.path !== undefined) {
                let handle = yield this.discovery.find('realms');
                if (!handle.address || !handle.port)
                    throw new Error('Unable to contact the RealmManagement Service service named realms');
                const accessTokenUrl = new URI(`http://${handle.address}:${handle.port}/`)
                    .segment('/realms')
                    .segment('/access_token').toString();
                const response = yield superagent.post(accessTokenUrl)
                    .send(request.body);
                return response.body;
            }
            const app_id = request.body["app_id"] || 'io.realm.Auth';
            const authResponse = yield this.authenticateOrCreateUser(request.body);
            const authToken = shared_2.signToken({ user_id: authResponse.userId, is_admin: authResponse.isAdmin }, this.privateKey);
            let expires = moment().add(10, 'm').unix();
            let json = {
                refresh_token: {
                    token: authToken,
                    token_data: {
                        app_id: app_id,
                        identity: authResponse.userId,
                        is_admin: authResponse.isAdmin,
                        access: ["refresh"],
                        expires: expires
                    }
                }
            };
            return json;
        });
    }
    getUser(userId) {
        return this.realm.objectForPrimaryKey('User', userId);
    }
    createOrUpdateUser(userId, isAdmin) {
        let created = this.getUser(userId) === undefined;
        let user;
        this.realm.write(() => {
            user = this.realm.create('User', {
                userId: userId,
                isAdmin: isAdmin
            }, true);
        });
        return { user, created };
    }
    getUserMetadata(userId) {
        return this.realm.objects('UserMetadataRow').filtered('user.userId = $0', userId).slice();
    }
    replaceUserMetadataByKey(userId, key, value) {
        const user = this.getUser(userId);
        if (!user) {
            throw new shared_1.JSONError({
                title: `userId of ${userId} does not exist`,
                code: 404,
                status: 404
            });
        }
        const row = user.metadata.filtered('key = $0 AND user.userId = $1', key, userId)[0];
        this.realm.write(() => {
            if (!row) {
                this.realm.create('UserMetadataRow', {
                    user: user,
                    key: key,
                    value: value
                });
            }
            else {
                row.value = value;
            }
        });
    }
    replaceUserMetadata(userId, metadatKeyValue) {
        const user = this.getUser(userId);
        if (!user) {
            throw new shared_1.JSONError({
                title: `userId of ${userId} does not exist`,
                code: 404,
                status: 404
            });
        }
        this.realm.write(() => {
            if (user.metadata) {
                this.realm.delete(user.metadata);
            }
            for (let key in metadatKeyValue) {
                this.realm.create('UserMetadata', {
                    user: user,
                    key: key,
                    value: metadatKeyValue[key]
                }, true);
            }
        });
    }
};
__decorate([
    decorators_1.InjectDiscovery,
    __metadata("design:type", Discovery_1.Discovery)
], AuthService.prototype, "discovery", void 0);
__decorate([
    decorators_1.InjectDiscovery,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Discovery_1.Discovery]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "discoveryReady", null);
__decorate([
    decorators_1.Post('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AuthService.prototype, "authenticate", null);
AuthService = __decorate([
    decorators_1.BaseRoute('/auth'),
    decorators_1.ServiceName('auth'),
    __metadata("design:paramtypes", [Object])
], AuthService);
exports.AuthService = AuthService;
//# sourceMappingURL=AuthService.js.map